@@ -9,8 +9,8 @@ extern "C" {
 #endif
 
 #define PD_MAJOR_VERSION 0
-#define PD_MINOR_VERSION 55
-#define PD_BUGFIX_VERSION 1
+#define PD_MINOR_VERSION 56
+#define PD_BUGFIX_VERSION 0
 #define PD_TEST_VERSION ""
 
 /* compile-time version check:
@@ -128,6 +128,7 @@ typedef unsigned __int64  uint64_t;
 
 EXTERN int pd_compatibilitylevel;   /* e.g., 43 for pd 0.43 compatibility */
 
+#define DEFAULTSRATE 48000      /* default audio sample rate */
 #define MAXPDSTRING 1000        /* use this for anything you want */
 #define MAXPDARG 5              /* max number of args we can typecheck today */
 
@@ -236,6 +237,9 @@ typedef struct _atom
     union word a_w;
 } t_atom;
 
+EXTERN_STRUCT _pdinstance;
+#define t_pdinstance struct _pdinstance
+
 EXTERN_STRUCT _class;
 #define t_class struct _class
 
@@ -404,14 +408,17 @@ EXTERN int binbuf_getnatom(const t_binbuf *x);
 EXTERN t_atom *binbuf_getvec(const t_binbuf *x);
 EXTERN int binbuf_resize(t_binbuf *x, int newsize);
 EXTERN void binbuf_eval(const t_binbuf *x, t_pd *target, int argc, const t_atom *argv);
+/* binbuf read/write flags */
+#define BINBUF_CR      (1<<0)
+#define BINBUF_SHEBANG (1<<1)
 EXTERN int binbuf_read(t_binbuf *b, const char *filename, const char *dirname,
-    int crflag);
+    int flag);
 EXTERN int binbuf_read_via_canvas(t_binbuf *b, const char *filename, const t_canvas *canvas,
-    int crflag);
+    int flag);
 EXTERN int binbuf_read_via_path(t_binbuf *b, const char *filename, const char *dirname,
-    int crflag);
+    int flag);
 EXTERN int binbuf_write(const t_binbuf *x, const char *filename, const char *dir,
-    int crflag);
+    int flag);
 EXTERN void binbuf_evalfile(t_symbol *name, t_symbol *dir);
 EXTERN t_symbol *binbuf_realizedollsym(t_symbol *s, int ac, const t_atom *av,
     int tonew);
@@ -437,6 +444,7 @@ EXTERN void pd_free(t_pd *x);
 EXTERN void pd_bind(t_pd *x, t_symbol *s);
 EXTERN void pd_unbind(t_pd *x, t_symbol *s);
 EXTERN t_pd *pd_findbyclass(t_symbol *s, const t_class *c);
+EXTERN t_pd *pd_findbyclassname(t_symbol *s, const t_symbol *classname);
 EXTERN void pd_pushsym(t_pd *x);
 EXTERN void pd_popsym(t_pd *x);
 EXTERN void pd_bang(t_pd *x);
@@ -539,6 +547,25 @@ EXTERN const t_parentwidgetbehavior *pd_getparentwidget(t_pd *x);
     automatically unless the CLASS_MULTICHANNEL flag is also set.
 */
 
+/* new names with pd_ prefix for class_new() etc.
+Use these if you want to protect against name clashes when
+using within libpd, for example when running in a VST.  If you only want
+to run in Pd as a standalone app the old names below will be OK.
+We only provide new names for the ones that seem likely to clash.  In the
+case of class_addbang, etc., the names are hidden by macros defined
+below.  We don't expect clashes there so we just define them back to the
+old names.  It's unclear where to stop with this name clash avoidance
+hooha, so here the choice is made to do as little as we seem to be able
+to get away with. */
+
+EXTERN t_class *pd_class_new(t_symbol *name, t_newmethod newmethod,
+    t_method freemethod, size_t size, int flags, t_atomtype arg1, ...);
+
+EXTERN void pd_class_addmethod(t_class *c, t_method fn, t_symbol *sel,
+    t_atomtype arg1, ...);
+
+/* ... and here are the traditional names for the same functions: */
+
 EXTERN t_class *class_new(t_symbol *name, t_newmethod newmethod,
     t_method freemethod, size_t size, int flags, t_atomtype arg1, ...);
 
@@ -604,6 +631,8 @@ EXTERN void class_setfreefn(t_class *c, t_classfreefn fn);
 
 /* ------------   printing --------------------------------- */
 
+/* post and pd_post are synonyms; pd_post is safer if dynamically linked */
+EXTERN void pd_post(const char *fmt, ...);
 EXTERN void post(const char *fmt, ...);
 EXTERN void startpost(const char *fmt, ...);
 EXTERN void poststring(const char *s);
@@ -648,11 +677,23 @@ EXTERN int sys_close(int fd);
 EXTERN FILE *sys_fopen(const char *filename, const char *mode);
 EXTERN int sys_fclose(FILE *stream);
 
-/* ------------  threading ------------------- */
+/* ------------- threading ------------------- */
+
+/* NB: do not use this in externals because it may deadlock!
+ * For a safe alternative see pd_queue_mess() */
 EXTERN void sys_lock(void);
 EXTERN void sys_unlock(void);
 EXTERN int sys_trylock(void);
 
+typedef void (*t_messfn)(t_pd *obj, void *data);
+/* send a message to a Pd object from another (helper) thread.
+ * 'fn' will be called on the scheduler thread with 'obj' and 'data'.
+ * If the message has been canceled, the 'obj' argument is NULL, see
+ * pd_queue_cancel() below. NB: do not forget to free the 'data' object! */
+EXTERN void pd_queue_mess(struct _pdinstance *instance, t_pd *obj, void *data, t_messfn fn);
+/* cancel all pending messages for the given object;
+ * typically called in the object destructor AFTER joining the helper thread. */
+EXTERN void pd_queue_cancel(t_pd *obj);
 
 /* --------------- signals ----------------------------------- */
 
@@ -1005,7 +1046,6 @@ struct _pdinstance
     int pd_islocked;
 #endif
 };
-#define t_pdinstance struct _pdinstance
 EXTERN t_pdinstance pd_maininstance;
 
 /* m_pd.c */
@@ -1063,6 +1103,18 @@ EXTERN t_symbol s_pointer, s_float, s_symbol, s_bang, s_list, s_anything,
   s_signal, s__N, s__X, s_x, s_y, s_;
 #endif
 
+#ifdef VST_CLEANSER
+/* LATER get rid of this, perhaps in 2035 .
+If we're in a VST plug-in and want to import plug-ins that use symbols like
+"s__X" directly, we need to provide them even though PDINSTANCE is set.  In
+this case we need to cleanse occurrences of the global symbols so that plug-ins
+don't bind to them or use them in messages.  So we expect the VST plug-in to
+provide those global symbols, along with a function to catch the extern using
+them and alias them to the per-instance version of the same symbol.  This should
+go along with a warning message that this will get removed someday. */
+void vst_cleanser(t_symbol **s);
+#endif
+
 EXTERN t_canvas *pd_getcanvaslist(void);
 EXTERN int pd_getdspstate(void);
 
